import sys
import os
import pwd
import grp
import subprocess

from contextlib import contextmanager
from charmhelpers.core.hookenv import log, ERROR, DEBUG
from charmhelpers.core import host

from charmhelpers.fetch import (
    apt_install, apt_update, add_source, filter_installed_packages
)


def run(command, exit_on_error=True, quiet=False):
    '''Run a command and return the output.'''
    if not quiet:
        log("Running {!r}".format(command), DEBUG)
    p = subprocess.Popen(
        command, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
        shell=isinstance(command, basestring))
    p.stdin.close()
    lines = []
    for line in p.stdout:
        if line:
            if not quiet:
                print line
            lines.append(line)
        elif p.poll() is not None:
            break

    p.wait()

    if p.returncode == 0:
        return '\n'.join(lines)

    if p.returncode != 0 and exit_on_error:
        log("ERROR: {}".format(p.returncode), ERROR)
        sys.exit(p.returncode)

    raise subprocess.CalledProcessError(
        p.returncode, command, '\n'.join(lines))


def chownr(path, owner, group):
    uid = pwd.getpwnam(owner).pw_uid
    gid = grp.getgrnam(group).gr_gid
    for root, dirs, files in os.walk(path):
        for momo in dirs:
            os.chown(os.path.join(root, momo), uid, gid)
            for momo in files:
                os.chown(os.path.join(root, momo), uid, gid)


@contextmanager
def chdir(d):
    cur = os.getcwd()
    try:
        yield os.chdir(d)
    finally:
        os.chdir(cur)


def prepare_cloudfoundry_environment(config_data, packages):
    if 'source' in config_data:
        add_source(config_data['source'], config_data.get('key'))
        apt_update(fatal=True)
    if packages:
        apt_install(packages=filter_installed_packages(packages), fatal=True)
    host.adduser('vcap')
