import os
import yaml

from charmhelpers.core import hookenv
from charmhelpers.contrib.openstack.context import OSContextGenerator


class RelationContext(OSContextGenerator):
    def __call__(self):
        if not hookenv.relation_ids(self.interface):
            return {}

        ctx = {}
        for rid in hookenv.relation_ids(self.interface):
            for unit in hookenv.related_units(rid):
                reldata = hookenv.relation_get(rid=rid, unit=unit)
                required = set(self.required_keys)
                if set(reldata.keys()).issuperset(required):
                    ns = ctx.setdefault(self.interface, {})
                    for k, v in reldata.items():
                        ns[k] = v
                    return ctx

        return {}


class ConfigContext(OSContextGenerator):
    def __call__(self):
        return hookenv.config()


class StorableContext(object):
    
    def store_context(self, file_name, config_data):
        with open(file_name, 'w') as file_stream:
            yaml.dump(config_data, file_stream)

    def read_context(self, file_name):
        with open(file_name, 'r') as file_stream:
            data = yaml.load(file_stream)
            if not data:
                raise OSError("%s is empty" % file_name)
            return data


# Stores `config_data` hash into yaml file with `file_name` as a name
# if `file_name` already exists, then it loads data from `file_name`.
class StoredContext(OSContextGenerator, StorableContext):

    def __init__(self, file_name, config_data):
        self.data = config_data
        if os.path.exists(file_name):
            self.data = self.read_context(file_name)
        else:
            self.store_context(file_name, config_data)
            self.data = config_data

    def __call__(self):
        return self.data


class StaticContext(OSContextGenerator):
    def __init__(self, data):
        self.data = data

    def __call__(self):
        return self.data


class NatsContext(RelationContext):
    interface = 'nats'
    required_keys = ['nats_port', 'nats_address', 'nats_user', 'nats_password']


class RouterContext(RelationContext):
    interface = 'router'
    required_keys = ['domain']


class LoggregatorContext(RelationContext):
    interface = 'loggregator'
    required_keys = ['shared_secret', 'loggregator_address']

