#!/usr/bin/env python

import os
import sys

from charmhelpers.core.hookenv import log

from charmhelpers.contrib.cloudfoundry import contexts
from charmhelpers.contrib.cloudfoundry import services
from config import *

from charmhelpers.core import hookenv
from charmhelpers.core.hookenv import log

hooks = hookenv.Hooks()

service_name, unit_number = os.environ['JUJU_UNIT_NAME'].split('/')
fileproperties = {'owner': 'vcap'}
services.register([
    {
        'service': LOGROUTER_JOB_NAME,
        'templates': [{
            'source': 'trafficcontroller.json',
            'target': LOGROUTER_CONFIG_PATH,
            'file_properties': fileproperties,
            'contexts': [
                contexts.StaticContext({'service_name': service_name}),
                contexts.RouterContext(),
                contexts.LoggregatorContext(),
                contexts.NatsContext()
            ]
        }]
    }], os.path.join(hookenv.charm_dir(), 'templates'))


@hooks.hook()
def start():
    """We start post nats and post logs relation."""


@hooks.hook()
def stop():
    services.stop_services()


@hooks.hook('config-changed')
def config_changed():
    pass


@hooks.hook('nats-relation-changed')
def nats_relation_changed():
    services.reconfigure_services()


@hooks.hook('router-relation-changed')
def router_relation_changed():
    services.reconfigure_services()


@hooks.hook('loggregator-relation-changed')
def loggregator_relation_changed():
    services.reconfigure_services()


if __name__ == '__main__':
    log("Running {} hook".format(sys.argv[0]))
    if hookenv.relation_id():
        log("Relation {} with {}".format(
            hookenv.relation_id(), hookenv.remote_unit()))
    hooks.execute(sys.argv)
