import json
import os

CONF_PATH = "/etc/vcap/trafficcontroller.json"


def update_log_config(**kw):
    service, unit_seq = os.environ['JUJU_UNIT_NAME'].split('/')
    data = {
        "Loggregators": {"z1": []},
        "Host": "0.0.0.0",
        "IncomingPort": 3456,
        "OutgoingPort": 8080,
        "Syslog": "",
        "Zone": "z1",
        "ApiHost": "http://ccng.example.cf-app.com",
        "SystemDomain": "example.cf-app.com",
        "NatsHost": None,
        "NatsPort": None,
        "NatsUser": None,
        "NatsPass": None,
        "VarzUser": service,
        "VarzPass": service,
        "VarzPort": 8888}

    if os.path.exists(CONF_PATH):
        with open(CONF_PATH) as fh:
            data.update(json.loads(fh.read()))

    previous = dict(data)

    # Config changes
    if 'domain' in kw:
        data['SystemDomain'] = kw['domain']
        data['ApiHost'] = "http://api.{}".format(kw['domain'])

    # Relation changes
    if 'nats_address' in kw:
        data['NatsHost'] = kw['nats_address']
        data['NatsPort'] = int(kw['nats_port'])
        data['NatsUser'] = kw['nats_user']
        data['NatsPass'] = kw['nats_password']

    if 'loggregators' in kw:
        data['Loggregators'] = {'z1': kw['loggregators']}

    # Write out new config if updated
    if data != previous:
        with open(CONF_PATH, 'w') as fh:
            fh.write(json.dumps(data, indent=2))
        # Signal we have changed and have a working config
        if data['NatsHost'] and data['Loggregators']['z1']:
            return True

    # Either no change or config not complete, ie. nothing to do.
    return False
